package samples;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.image.BufferedImage;

import org.jdesktop.lg3d.sg.Appearance;
import org.jdesktop.lg3d.sg.Texture;
import org.jdesktop.lg3d.sg.TransparencyAttributes;
import org.jdesktop.lg3d.sg.utils.image.TextureLoader;
import org.jdesktop.lg3d.utils.action.ActionNoArg;
import org.jdesktop.lg3d.utils.action.ColorAlphaChangeAction;
import org.jdesktop.lg3d.utils.action.ScaleAction;
import org.jdesktop.lg3d.utils.eventaction.ComponentMover;
import org.jdesktop.lg3d.utils.eventadapter.MouseClickedEventAdapter;
import org.jdesktop.lg3d.utils.eventadapter.MouseEnteredEventAdapter;
import org.jdesktop.lg3d.utils.shape.FuzzyEdgePanel;
import org.jdesktop.lg3d.utils.shape.SimpleAppearance;
import org.jdesktop.lg3d.wg.Component3D;
import org.jdesktop.lg3d.wg.Cursor3D;
import org.jdesktop.lg3d.wg.Frame3D;
import org.jdesktop.lg3d.wg.event.LgEventSource;
import org.jdesktop.lg3d.wg.event.MouseEvent3D;

public class HelloWorld6 {

    // [gϊp 0.0254 [inch/m] / 72.0f [pixel/inch]
    private static final float UNIT_TRANS_FACTOR = 0.0254f / 72.0f;
    private static final float DEPTH = 0.005f;
    private static final float EDGE = 0.0f;
    
    public HelloWorld6() {
        Frame3D frame3d = new Frame3D();
        
        BufferedImage image = createImage();

        TextureLoader loader = new TextureLoader(image);
        Texture texture =  loader.getTexture();

        Appearance appearance = new SimpleAppearance(1.0f, 1.0f, 1.0f, 1.0f,
                                                     SimpleAppearance.ENABLE_TEXTURE 
                                                     | SimpleAppearance.DISABLE_CULLING);
        appearance.setTexture(texture);
            
        float width = texture.getWidth() * UNIT_TRANS_FACTOR;
        float height = texture.getHeight() * UNIT_TRANS_FACTOR;
        
        FuzzyEdgePanel panel = new FuzzyEdgePanel(width, height, EDGE, appearance);
        
        Component3D comp3d = new Component3D();
        comp3d.addChild(panel);
        frame3d.addChild(comp3d);
        
        frame3d.setSize(width * 1.1f, height * 1.1f, DEPTH);
        
        doEnableToMove(comp3d);
        doEnableToChangeAlpha(comp3d, appearance);
        doEnableToChangeSize(comp3d);
        doEnabltToExit(frame3d, comp3d);
        
        frame3d.setCapabilities();
        frame3d.setActive(true);
        frame3d.setVisible(true);
    }

    private BufferedImage createImage() {
        // C[W̍쐬
        BufferedImage image = new BufferedImage(200, 25, BufferedImage.TYPE_INT_ARGB);
        Graphics2D g = (Graphics2D)image.getGraphics();

        g.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, 
                           RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
        g.setColor(new Color(192, 64, 64));
        g.setFont(new Font("Serif", Font.BOLD, 28));
        g.drawString("Hello, World!", 0, 20);
        g.dispose();

        return image;
    }

    private void doEnableToMove(Component3D comp3d) {
        // Component3D ړł悤ɂ
        new ComponentMover(comp3d);

        // J[\̕ύX
        comp3d.setCursor(Cursor3D.MOVE_CURSOR);
    }

    private void doEnableToChangeAlpha(Component3D comp3d, Appearance appearance) {
        // Appearance 瓧x̃Agr[go
        TransparencyAttributes attribute = appearance.getTransparencyAttributes();
        
        // Mouse  panel ɓ瓧xω
        // ĂȂƂ  ... alpha = 0.5
        // Ƃ        ... alpha = 1.0
        new MouseEnteredEventAdapter(
            comp3d,
            new ColorAlphaChangeAction(comp3d, attribute, 0.5f, 1.0f, 500));
    }

    private void doEnableToChangeSize(Component3D comp3d) {
        // Mouse  panel ɓTCYg傷
        new MouseEnteredEventAdapter(
            comp3d,
            new ScaleAction(comp3d, 1.1f, 500));
    }

    private void doEnabltToExit(final Frame3D frame3d, Component3D comp3d) {
        new MouseClickedEventAdapter(
            comp3d,
            new ActionNoArg() {
                public void performAction(LgEventSource source) {
                    System.out.println("PerformAction3");
                    frame3d.changeActivity(false);
                }
            },
            MouseEvent3D.ButtonId.BUTTON3);
    }

    public static void main(String[] args) {
        new HelloWorld6();
    }
}
