package samples;

import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Timer;
import java.util.TimerTask;

import javax.imageio.ImageIO;

import org.jdesktop.lg3d.sg.Appearance;
import org.jdesktop.lg3d.sg.Shape3D;
import org.jdesktop.lg3d.sg.Texture;
import org.jdesktop.lg3d.sg.utils.image.TextureLoader;
import org.jdesktop.lg3d.utils.action.ActionNoArg;
import org.jdesktop.lg3d.utils.eventaction.ComponentMover;
import org.jdesktop.lg3d.utils.eventadapter.MouseClickedEventAdapter;
import org.jdesktop.lg3d.utils.shape.FuzzyEdgePanel;
import org.jdesktop.lg3d.utils.shape.SimpleAppearance;
import org.jdesktop.lg3d.wg.Component3D;
import org.jdesktop.lg3d.wg.Cursor3D;
import org.jdesktop.lg3d.wg.Frame3D;
import org.jdesktop.lg3d.wg.event.LgEventSource;
import org.jdesktop.lg3d.wg.event.MouseEvent3D;

public class Juggler {

    // [gϊp 0.0254 [inch/m] / 72.0f [pixel/inch]
    private static final float UNIT_TRANS_FACTOR = 0.0254f / 72.0f;
    private static final int WIDTH = 150;
    private static final int HEIGHT = 150;
    private static final float DEPTH = 0.005f;
    private static final float EDGE = 0.0f;

    private Appearance appearance;
    private List<Texture> textures;
    private int index;
    
    public Juggler() {
        Frame3D frame3d = new Frame3D();
        
        appearance = new SimpleAppearance(1.0f, 1.0f, 1.0f, 1.0f,
                                          SimpleAppearance.ENABLE_TEXTURE 
                                          | SimpleAppearance.DISABLE_CULLING);
        appearance.setCapability(Appearance.ALLOW_TEXTURE_WRITE);
        
        float width = WIDTH * UNIT_TRANS_FACTOR;
        float height = HEIGHT * UNIT_TRANS_FACTOR;
        
        Shape3D panel = new FuzzyEdgePanel(width, height, EDGE, appearance);
        panel.setCapability(Shape3D.ALLOW_APPEARANCE_READ);
        
        Component3D comp3d = new Component3D();
        comp3d.addChild(panel);
        
        frame3d.addChild(comp3d);
        frame3d.setSize(width * 1.2f, height * 1.2f, DEPTH);
        
        doEnableToMove(comp3d);
        doEnabltToExit(frame3d, comp3d);

        try {
            createTexture();
            startAnimation();
        } catch (IOException ex) {
            ex.printStackTrace();
            return;
        }
        
        frame3d.setCapabilities();
        frame3d.setActive(true);
        frame3d.setVisible(true);
    }

    public void createTexture() throws IOException {
        textures = new ArrayList<Texture>();

        for (int i = 0; i < 4; i++) {
            BufferedImage image = ImageIO.read(new File("juggler" + i + ".png"));
            TextureLoader loader = new TextureLoader(image);
            Texture texture = loader.getTexture();
            textures.add(texture);
        }
    }

    public void startAnimation() {
        TimerTask task = new TimerTask() {
                public void run() {
                    appearance.setTexture(textures.get(index));
                    
                    index++;
                    if (index >= textures.size()) {
                        index = 0;
                    }
                }
            };

        Timer timer = new Timer();
        timer.scheduleAtFixedRate(task, 0L, 100L);
    }

    private void doEnableToMove(Component3D comp3d) {
        // Component3D ړł悤ɂ
        new ComponentMover(comp3d);

        // J[\̕ύX
        comp3d.setCursor(Cursor3D.MOVE_CURSOR);
    }

    private void doEnabltToExit(final Frame3D frame3d, Component3D comp3d) {
        new MouseClickedEventAdapter(
            comp3d,
            new ActionNoArg() {
                public void performAction(LgEventSource source) {
                    System.out.println("PerformAction3");
                    frame3d.changeActivity(false);
                }
            },
            MouseEvent3D.ButtonId.BUTTON3);
    }

    public static void main(String[] args) {
        new Juggler();
    }
}
